#####################################################################################
#
#		:::[  T h e   R u s s i a n   E g g d r o p  R e s o u r c e  ]::: 
#      ____                __                                                      
#     / __/___ _ ___ _ ___/ /____ ___   ___      ___   ____ ___ _     ____ __ __   
#    / _/ / _ `// _ `// _  // __// _ \ / _ \    / _ \ / __// _ `/    / __// // /   
#   /___/ \_, / \_, / \_,_//_/   \___// .__/ __ \___//_/   \_, / __ /_/   \___/    
#        /___/ /___/                 /_/    /_/           /___/ /_/                
#
#
#####################################################################################
#
# poisk.tcl 3.6.1
#
# Description:
#	eggdrop frontend for popular search engines
#
# Authors: Shrike <shrike@eggdrop.org.ru>
#
# Official support: irc.eggdrop.org.ru @ #eggdrop
# 
#####################################################################################
#
# Version History:
#	
#  		SHR - Shrike
#	
#		Who		DATE	Changes
#   	---     ------  -------------------------------------------------------------
# v3.6.1
#		SHR		050613	Fixed !rambler, !yandex and !google
#						Heavy optimized !rambler, !yandex, !google and !msg for
#						low traffic usage
#
# v3.6
#		SHR		041213	Fixed minor bug in !rambler
#						Rewriten bind part, to make available on the fly conversion
#						of binds to koi8
#						Added automatic translation of messages to koi8-r
#						Improves vd_patch support. 
# v3.5.2
#
#		SHR		041212	Fixed !crack. 'total' in no more available on
#						site - crackit.ru
#						Fixed !xdcc. Site is moved.
#						Added search engine: search.msn.com
# v3.5.1
#
#		SHR		041013	Fixed displaying of 'total'
#
# v3.5
#
#		SHR		040616	Removed old socket functions
#						Rewrited to use socket functions in egglib_pub
#						Added proxy support
#
#   	SHR		040615	Added this header ;)
#						Refactored for namespace usage					
#	
#####################################################################################
# Last version of egglib_pub can be grabbed here:
# http://eggdrop.org.ru/scripts/egglib.zip

if { ![info exists egglib(ver)] || [expr {$egglib(ver) < 1.2}] } {
	putlog "***********************************************"
	putlog "   egglib_pub NOT FOUND OR VERSION IS TOO OLD!"
	putlog "   Download last version of egglib_pub here:"
	putlog "  http://eggdrop.org.ru/scripts/egglib_pub.zip"
	putlog "***********************************************"
	return
}

namespace eval poisk {}

setudef flag nopubpoisk

foreach p [array names poisk *] { catch {unset poisk($p) } }

set poisk(timeout) 	20
set poisk(debug) 	0

#    cp1251  -   1,  koi8 -  0
#     
set poisk(cp1251)   1

set poisk(pubcmdchar) "!"
set poisk(msgcmdchar) "!"
set poisk(bindflag) "-"

set poisk(use_proxy) 0
set poisk(proxy_host) "proxy1.bezeqint.net"
set poisk(proxy_port) 3128

set poisk(ver) 		"3.6.1"
set poisk(authors) 	"Shrike <shrike@eggdrop.org.ru>"

set poisk(txt,help) "\[-\] <>"
set poisk(txt,err_google) "    google.ru... -.-"
set poisk(txt,err_yandex) "    yandex.ru... -.-"
set poisk(txt,err_rambler) "    rambler.ru... -.-"
set poisk(txt,err_crack) "    crackit.ru... -.-"
set poisk(txt,err_msn) "    search.msn.com... -.-"
set poisk(txt,err_image) "    images.google.com... -.-"
set poisk(txt,err_file) "    www.filesearch.ru... -.-"
set poisk(txt,err_xdcc) "    www.mydownloader.com... -.-"
set poisk(txt,fail_google) "   google.ru    ... -.-"
set poisk(txt,fail_yandex) "   yandex.ru    ... -.-"
set poisk(txt,fail_rambler) "   rambler.ru    ... -.-"
set poisk(txt,fail_crack) "   crackit.ru    ... -.-"
set poisk(txt,fail_msn) "   search.msn.com    ... -.-"
set poisk(txt,fail_image) "   images.google.com    ... -.-"
set poisk(txt,fail_file) "   www.filesearch.ru    ... -.-"
set poisk(txt,fail_xdcc) "   www.mydownloader.com    ... -.-"
set poisk(txt,total) ":"
set poisk(txt,wait) "    ..."

#    (thanks to Twin @ Rusnet)
foreach bind [binds "::poisk::*"] {
    catch {unbind [lindex $bind 0] [lindex $bind 1] [lindex $bind 2] [lindex $bind 4]}
}

#   (, )
set poisk(binds) {
	{{google} {google}}	{{} {google}} {{} {google}} {{poisk} {google}} 
	{{yandex} {yandex}} {{} {yandex}} {{ya} {yandex}}
	{{rambler} {rambler}} {{} {rambler}} {{ra} {rambler}}
	{{image} {image}} {{} {image}} {{} {image}}
	{{file} {file}} {{} {file}} {{files} {file}}
	{{linux} {linux}} {{} {linux}} {{} {linux}}
	{{bsd} {bsd}} {{} {bsd}} {{} {bsd}}
	{{mac} {mac}} {{} {mac}} {{macintosh} {mac}}
	{{microsoft} {ms}}
	{{crack} {crack}} {{} {crack}} {{keygen} {crack}} {{} {crack}} {{} {crack}}
	{{xdcc} {xdcc}} {{} {xdcc}}
	{{msn} {msn}} {{} {msn}} {{ms} {msn}}
}

#  
foreach b $poisk(binds) {
	set bn [lindex $b 0]
	set pr [lindex $b 1]
	if { $poisk(cp1251) == 0 } {
		set bn [encoding convertto koi8-r [encoding convertfrom cp1251 $bn]]
		#putlog "$bn"
	}
	bind pub $poisk(bindflag) $poisk(pubcmdchar)$bn [list ::poisk::pub_$pr $pr]
	bind msg $poisk(bindflag) $poisk(msgcmdchar)$bn [list ::poisk::msg_$pr $pr]
	proc ::poisk::msg_$pr {param nick uhost hand args} 	{ if { [llength [split [lindex $args 0] ] ] > 1 || [llength [split [lindex $args 0] ] ] == 0 } { set args [split [lindex $args 0]] }; ::poisk::search $nick $uhost $nick $nick $args "$param" "$param" }
	proc ::poisk::pub_$pr {param nick uhost hand chan args} 	{ if {[channel get $chan nopubpoisk]} return; if { [llength [split [lindex $args 0] ] ] > 1 || [llength [split [lindex $args 0] ] ] == 0 } { set args [split [lindex $args 0]] }; ::poisk::search $nick $uhost $hand $chan $args "$param" "$param" }
	proc ::poisk::${pr}_on_error {id nick chan uhost com} { global poisk; catch { unset poisk(host,$uhost) }; ::egglib::out $nick $chan $poisk(txt,err_$com) }
}

#       koi8-r
foreach p [array names poisk txt*] {
	if { $poisk(cp1251) == 0 } {
		set poisk($p) [encoding convertto koi8-r [encoding convertfrom cp1251 $poisk($p)]]
		#putlog "$poisk($p)"
	}
}

########################################################
proc ::poisk::search { nick uhost hand chan args t com } {
 	global poisk
 	if { [llength [split [lindex $args 0] ] ] > 1 || [llength [split [lindex $args 0] ] ] == 0 } { set args [split [lindex $args 0]] }
	if {[llength $args]==0} { ::egglib::outh $nick $chan "!$com" $poisk(txt,help); return	}
	::egglib::log $chan $nick "$com" "$args"
	set query ""; set farg 0; set qnum 1
	if { [regexp -nocase -- {^-(\d+)$} [string trim [lindex $args 0]] tr qnum] } {
		set farg 1; if { $qnum > 100 } { set qnum 100 }; if { $qnum == 0 } { set qnum 1 }
	}
	
	#     .
	for { set index $farg } { $index<[llength $args] } { incr index } {
		set query "$query[lindex $args $index]"
		if {$index<[llength $args]-1} {
			set query "$query+"
		}
	}
	
	#         
	if { $poisk(cp1251) } {
		if { [string match $com "msn"] } {
			set query [encoding convertto utf-8 [encoding convertfrom cp1251 $query]]
		}
	} else {
		if { [string match $com "msn"] } {
			set query [encoding convertto utf-8 [encoding convertfrom koi8-r $query]]
		} else {
			set query [encoding convertto cp1251 [encoding convertfrom koi8-r $query]]
		}	
	}
	
	set query [::egglib::urlencode $query]
	
	switch $com {
		crack 	{ set query "http://crackit.ru/?search=$query" }
		google 	{ incr qnum -1; set query "http://www.google.ru/pda?sourceid=navclient&ie=win-1251&oe=win-1251&hl=ru&start=$qnum&num=1&q=$query" }
		yandex	{ set query "http://www.yandex.ru/yandsearch?stype=&nl=0&numdoc=$qnum&text=$query" }
		rambler { set query "http://search.rambler.ru/srch?start=$qnum&limit=1&words=$query" }
		image	{ incr qnum -1; set query "http://images.google.com/images?sourceid=navclient&ie=win-1251&oe=win-1251&num=$qnum&hl=ru&q=$query" }
		file	{ set query "http://www.filesearch.ru/cgi-bin/s?t=f&w=a&q=$query" }
		xdcc	{ set query "http://www.mydownloader.com//search.asp?searchstring=$query&submit=xdcc+search" }
		msn		{ set query "http://search.msn.com/results.aspx?q=$query&format=rss&first=$qnum&count=1&FORM=R0RE" }
		default	{ incr qnum -1; set query "http://www.google.ru/$t?sourceid=navclient&ie=win-1251&oe=win-1251&hl=ru&start=$qnum&num=1&q=$query";set com "google" }
	}

	if {[info exists poisk(host,$uhost)]} { ::egglib::outn $nick $poisk(txt,wait); return	}
	set poisk(host,$uhost) $qnum
	
	set id [::egglib::http_init "::poisk::$com\_"]
	if { $poisk(use_proxy) } {
		::egglib::http_set_proxy $id $poisk(proxy_host) $poisk(proxy_port)
	}
	::egglib::http_set_timeout $id $poisk(timeout)
  	::egglib::http_get $id $query [list $nick $chan $uhost $com]
}
proc ::poisk::google_on_data {id html nick chan uhost com} {
	global poisk
	
	regsub -all -- \n $html {} html; regsub -all -nocase -- </a> $html </a>\n html 
	catch { unset poisk(host,$uhost) }
	#::egglib::writedata "debug.htm" [list $html]
	if { $poisk(debug) } { putlog "\[poisk\] callback executed... analyzing data..." }
  	set res "";	set total ""; set desc ""
	foreach line [split $html \n] {
  		if { ![regexp -nocase -- {of\ about\ <b>(.*?)</b>\ for} $line garb total] } { 
  			if { ![regexp -nocase -- {\ <b>(.*?)</b>\ } $line garb total] } { 
  				if { ![regexp -nocase -- {\ \ <b>(.*?)</b>\.} $line garb total] } {
  					 if { ![regexp -nocase -- {\d+?\ \ (.*?)\ } $line garb total] } {
  					 	regexp -nocase -- {\ \ (.*?)\.<br />} $line garb total
  					 }
  				}
  			} 
  		}
		if { [regexp -nocase -- {<p\ class=g><a\ href=(.*?)>(.*)</a>} $line garb res desc] || 
			 [regexp -nocase -- {<p\ class=g><a\ href=(.*?)/>} $line garb res] ||
			 [regexp -nocase -- {<p><a\ href=/.*?\&q=(.*?)>(.*)</a>} $line garb res desc] ||
			 [regexp -nocase -- {<b>\d+</b> <a\ href=\"(.*?)\">(.*)</a>} $line garb res desc] ||
			 [regexp -nocase -- {<p><a\ href=(.*?)>(.*)</a>} $line garb res desc] } { 
			 	break
		}
	}
	if { $poisk(debug) } { putlog "\[poisk\] res=$res total=$total desc=$desc" }
	if { $res == "" || [string equal $res "webhp"] } { ::egglib::out $nick $chan $poisk(txt,fail_$com); return }
	set desc1 [encoding convertto iso8859-1 $desc] 
	set desc1 [encoding convertfrom cp1251 $desc1] 
	if { $poisk(cp1251) } {
		set desc [encoding convertto cp1251 $desc1] 
	} else {
		set desc [encoding convertto koi8-r $desc1] 
	}
	set res [string trim $res]
	set desc [string trim [::egglib::unhtml $desc]]
	set total [string trim [::egglib::unhtml $total]]
	if { $total != "" } { set total " ($poisk(txt,total) $total)" }
	if { $desc != "" } { set desc " - \0035$desc\003" }
	
	::egglib::out $nick $chan "\00312$res\003$desc$total"
}

proc ::poisk::rambler_on_data {id html nick chan uhost com} {
	global poisk
	#::egglib::writedata "debug.htm" [list $html]
	regsub -all -- \n $html {} html; regsub -all -nocase -- </a> $html </a>\n html 
	catch { unset poisk(host,$uhost) }
	if { $poisk(debug) } { putlog "\[poisk\] callback executed... analyzing data..." }
  	set recode 0; set res ""; set total ""; set desc ""
	foreach line [split $html \n] {	if { ![regexp -nocase -- {\,\ :&nbsp;<b>(.*?)</b>} $line garb total] } { if {[regexp -nocase -- {\,\ :&nbsp;<b>(.*?)</b>} $line garb total] } { set recode 1 } }
		if { [regexp -nocase -- {<div\ class=\"ttl\"><a\ .*?href=\"(.*?)\".*?>(.*?)</a>} $line garb res desc] || 
			 [regexp -nocase -- {<a\ class=ttl.*?href=\"(.*?)\"} $line garb res] } { break }
	}
	set desc1 [encoding convertto iso8859-1 $desc] 
	if { $recode == 1 } {
		set desc1 [encoding convertfrom koi8-r $desc1]
	} else {
		set desc1 [encoding convertfrom cp1251 $desc1]
	} 
	if { $poisk(cp1251) } {
		set desc [encoding convertto cp1251 $desc1] 
	} else {
		set desc [encoding convertto koi8-r $desc1] 
	}
	if { $poisk(debug) } { putlog "\[poisk\] res=$res total=$total desc=$desc recode=$recode" }
	if { $res == "" } { ::egglib::out $nick $chan $poisk(txt,fail_$com); return }
	set desc [string trim [::egglib::unhtml $desc]]
	set total [string trim [::egglib::unhtml $total]]
	if { $total != "" } { set total " ($poisk(txt,total) $total)" }
	if { $desc != "" } { set desc " - \0035$desc\003" }
	::egglib::out $nick $chan "\00312$res\003$desc$total"
}

proc ::poisk::yandex_on_data {id html nick chan uhost com} {
	global poisk
	#::egglib::writedata "debug.htm" [list $html]
	regsub -all -- \n $html {} html; regsub -all -nocase -- </a> $html </a>\n html 
	set qnum $poisk(host,$uhost); set num 0
	catch { unset poisk(host,$uhost) }
	if { $poisk(debug) } { putlog "\[poisk\] callback executed... analyzing data..." }
  	set res "";	set total ""; set desc ""
	foreach line [split $html \n] {
  		regexp -nocase -- { :\ &nbsp;&\#151;\ <b>(.*?)</b>\,} $line garb total
		if { [regexp -nocase -- {\'web\'\)\" href=\"(.*?)\" target=_blank>(.*?)</A>} $line garb res desc] || 
			 [regexp -nocase -- {\'web\'\)\" href=\"(.*?)\" target=_blank} $line garb res] } { incr num; if { $num != $qnum } { set res ""; set desc "" } else { break } }
	}
	if { $poisk(debug) } { putlog "\[poisk\] res=$res desc=$desc total=$total" }
	if { $res == "" } { ::egglib::out $nick $chan $poisk(txt,fail_$com); return }
	set desc1 [encoding convertto iso8859-1 $desc];	set desc1 [encoding convertfrom cp1251 $desc1] 
	if { $poisk(cp1251) } {	set desc [encoding convertto cp1251 $desc1] } else { set desc [encoding convertto koi8-r $desc1] }
	set desc [string trim [::egglib::unhtml $desc]]
	set total [string trim [::egglib::unhtml $total]]
	if { $total != "" } { set total " ($poisk(txt,total) $total)" }
	if { $desc != "" } { set desc " - \0035$desc\003" }
	::egglib::out $nick $chan "\00312$res\003$desc$total"
}

proc ::poisk::crack_on_data {id html nick chan uhost com} {
	global poisk
	#::egglib::writedata "debug.htm" [list $html]
	set qnum $poisk(host,$uhost); set num 0
	regsub -all -- \n $html {} html; regsub -all -nocase -- </a> $html </a>\n html 
	catch { unset poisk(host,$uhost) }
	if { $poisk(debug) } { putlog "\[poisk\] callback executed... analyzing data... (req: $qnum)" }
  	set res ""; set desc ""
  	foreach line [split $html \n] {
  		if { [regexp -nocase -- {<font\ class=textnorm>.*?<a\ href=\'(.*?)\'>(.*?)</a>} $line garb res desc] || [regexp -nocase -- {<font\ class=textnorm>.*?<a\ href=\'(.*?)\'>} $line garb res] || [regexp -nocase -- {<br>\d+\.\ <a\ href=\"(.*?)\">(.*?)</a>} $line garb res desc] } { incr num; if { $num != $qnum } { set res ""; set desc "" } else { break } }
	}
	set desc1 [encoding convertto iso8859-1 $desc];	set desc1 [encoding convertfrom cp1251 $desc1] 
	if { $poisk(cp1251) } {	set desc [encoding convertto cp1251 $desc1] } else { set desc [encoding convertto koi8-r $desc1] }
	if { $poisk(debug) } { putlog "\[poisk\] res=$res desc=$desc" }
	if { $res == "" } { ::egglib::out $nick $chan $poisk(txt,fail_$com); return }
	set desc [string trim [::egglib::unhtml $desc]]
	if { $desc != "" } { set desc " - \0035$desc\003" }
	::egglib::out $nick $chan "\00312$res\003$desc"
}

proc ::poisk::file_on_data {id html nick chan uhost com} {
	global poisk
	#::egglib::writedata "debug.htm" [list $html]
	catch { unset poisk(host,$uhost) }
	if { $poisk(debug) } { putlog "\[poisk\] callback executed... analyzing data..." }
  	set res ""
  	foreach line [split $html \n] {
  		if { [regexp -nocase -- {class=lg>.*?</a><a\ href=(.*?)\ class=lf>.*?</a>} $line garb res] } {
			break
		} elseif { [regexp -nocase -- {</a><a\ href=/cgi-bin/s\?t=n&q=(.*?)\ class=lg>(.*?)</a>$} $line garb res] } {
			break
		}
	}
	if { $poisk(debug) } { putlog "\[poisk\] res=$res" }
	if { $res == "" } { ::egglib::out $nick $chan $poisk(txt,fail_$com); return }
	::egglib::out $nick $chan "\00312$res\003"
}

proc ::poisk::image_on_data {id html nick chan uhost com} {
	global poisk
	set qnum $poisk(host,$uhost); set num 0; set inum 0
	catch { unset poisk(host,$uhost) }
	regsub -all -- \n $html {} html
	regsub -all -nocase -- {\?imgurl} $html \n\?imgurl html; 
	regsub -all -nocase -- {<font\ size=-1>} $html \n<font\ size=-1> html
	if { $poisk(debug) } { putlog "\[poisk\] callback executed... analyzing data... (req: $qnum)" }
  	set res "";	set total ""; set size ""
  	foreach line [split $html \n] {
  		if { ![regexp -nocase -- {of\ about\ <b>(.*?)</b>\ for} $line garb total] } { if { ![regexp -nocase -- {\ <b>(.*?)</b>\ } $line garb total] } { regexp -nocase -- {\ \ <b>(.*?)</b>\.} $line garb total } }
		if { [regexp -nocase -- {\?imgurl=(.*?)\&imgrefurl=} $line garb ires] || [regexp -nocase -- {\?imgurl=(.*?)\&imgrefurl=} $line garb ires] } { incr inum; if { $inum == $qnum } { set res $ires } }
		if { [regexp -nocase -- {<font\ size=-1>(.*?)\ pixels} $line garb size] || [regexp -nocase -- {<font\ size=-1>(.*?)\ } $line garb size] } { incr num; if { $num != $qnum } { set size "" } else { break } }
	}
	if { $poisk(debug) } { putlog "\[poisk\] res=$res total=$total size=$size" }
	if { $res == "" } { ::egglib::out $nick $chan $poisk(txt,fail_$com); return }
	set total [string trim [::egglib::unhtml $total]]
	set size [string trim [::egglib::unhtml $size]]
	if { $total != "" } { set total " ($poisk(txt,total) $total)" }
	if { $size != "" } { set size " - \0035\002\002$size\003" }
	::egglib::out $nick $chan "\00312$res\003$size$total"
}

proc ::poisk::xdcc_on_data {id html nick chan uhost com} {
	global poisk
	set qnum $poisk(host,$uhost); set num 0
	regsub -all -- {\n} $html {} html
	regsub -all -nocase -- <tr><td\ class=tabletitle> $html \n<tr><td\ class=tabletitle> html
	regsub -all -nocase -- {<a\ title=\"Copy\ \'} $html \n<a\ title=\"Copy\ \' html
	#::egglib::writedata "debug.htm" [list $html]
	catch { unset poisk(host,$uhost) }
	if { $poisk(debug) } { putlog "\[poisk\] callback executed... analyzing data... (req: $qnum)" }
  	set res "";	set hit ""; set fsize ""; set desc ""
	foreach line [split $html \n] {
  		if { [regexp -nocase -- {toclip\(\'(/msg\ .*?)\'\).*?</a></td><td class=table>.*?</td><td\ class=table>.*?</td><td\ class=table>(.*?)</td><td\ class=table>.*?</td><td\ class=table>(.*?)</td>.*?href=(.*?)\ class=url>} $line garb hit fsize desc res] } {
			incr num
			if { $num != $qnum } { set res ""; set desc ""; set hit ""; set fsize "" } else { break }
		}
	}
	set desc1 [encoding convertto iso8859-1 $desc];	set desc1 [encoding convertfrom cp1251 $desc1] 
	if { $poisk(cp1251) } {	set desc [encoding convertto cp1251 $desc1] } else { set desc [encoding convertto koi8-r $desc1] }
	if { $poisk(debug) } { putlog "\[poisk\] res=$res hit=$hit fsize=$fsize desc=$desc" }
	if { $res == "" } { ::egglib::out $nick $chan $poisk(txt,fail_$com); return }
	set desc [string trim [::egglib::unhtml $desc]]
	if { $fsize != "" } { set size " - $desc ($fsize)"; set desc $hit }
	if { $desc != "" } { set desc " - \0035$desc\003" }
	::egglib::out $nick $chan "\00312$res\003$desc$size"
}

proc ::poisk::msn_on_data {id html nick chan uhost com} {
	global poisk
	regsub -all -- \n $html {} html; regsub -all -nocase -- <item> $html \n<item> html
	catch { unset poisk(host,$uhost) }
	#::egglib::writedata "debug.htm" [list $html]
	if { $poisk(debug) } { putlog "\[poisk\] callback executed... analyzing data... " }
  	set res ""; set desc ""
	foreach line [split $html \n] {
		if { [regexp -nocase -- {<item><title>(.*?)</title><link>(.*?)</link>} $line garb desc res] } { 
			break
		}
	}
	
	if { $poisk(debug) } { putlog "\[poisk\] res=$res desc=$desc" }
	if { $res == "" } { ::egglib::out $nick $chan $poisk(txt,fail_$com); return }
	set res [string trim $res]
	set desc1 [encoding convertfrom utf-8 $desc]
	if { $poisk(cp1251) } {
		set desc [encoding convertto cp1251 $desc1]
	} {
		set desc [encoding convertto koi8-r $desc1]
	}
	set desc [string trim [::egglib::unhtml $desc]]
	if { $desc != "" } { set desc " - \0035$desc\003" }
	::egglib::out $nick $chan "\00312$res\003$desc"
}

putlog "poisk.tcl $poisk(ver) by $poisk(authors) loaded"
